# Common configuration for all platforms and targets
# Initial verion by Anthony Truchet (Logilab SA) for CEREMA

if (${CMAKE_VERSION} VERSION_GREATER 2.8.4)
  # This is necessary as of CMake version 2.8.8
  cmake_policy(SET CMP0017 NEW)
  set(CMAKE_ECLIPSE_MAKE_ARGUMENTS "-j8")
endif(${CMAKE_VERSION} VERSION_GREATER 2.8.4)


# Enable using Solution folder to visualy group targets
set_property(GLOBAL PROPERTY USE_FOLDERS ON)
set(CURRENT_FOLDER "")
set_property(GLOBAL PROPERTY PREDEFINED_TARGETS_FOLDER "CMake Targets")


# Detect native MSVC/windows or GCC/Linux supported build or report error.
if(WIN32)
  if(CYGWIN)
    message(FATAL "Cygwin emulation environment will NOT work.")
  endif(CYGWIN)
  if(MINGW OR MSYS)
    message(WARNING "MINGW/MSYS is not supported : it might happen to work... or not.")
  endif(MINGW OR MSYS)
  if(NOT MSVC)
    message(WARNING "Compilation on Windows with something else than MS toolchain is not supported : it might happen to work... or not.")
  endif(NOT MSVC)
  set(SYS_NATIVE_WIN TRUE)
else(WIN32)
  if(NOT (UNIX AND CMAKE_COMPILER_IS_GNUCXX))
  message(FATAL "On UNIXes (and esp. Linux) only G++-compatible toolchain is supported : seem your system does not match...")
  endif(NOT (UNIX AND CMAKE_COMPILER_IS_GNUCXX))
  if(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
    message(WARNING "Mac OS X / g++ is not officially supported (yet) but might work...")
    set(SYS_MACOSX TRUE)
  endif(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
  if(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
    set(SYS_LINUX TRUE)
  endif(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
endif(WIN32)


# Set a default build type for single-configuration
# CMake generators if no build type is set.
if(NOT CMAKE_CONFIGURATION_TYPES AND NOT CMAKE_BUILD_TYPE)
   set(CMAKE_BUILD_TYPE Debug
       CACHE STRING "Build configuration" FORCE)
endif(NOT CMAKE_CONFIGURATION_TYPES AND NOT CMAKE_BUILD_TYPE)


if(MSVC)
  # Put here Visual Studio specific stuff
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}")
endif(MSVC)

if(CMAKE_COMPILER_IS_GNUCXX)
  # Put here GCC/G++ specific stuff
  set(WARNINGS_SETTINGS "-Wall -Wno-unknown-pragmas")
  set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} --std=c++03 ${WARNINGS_SETTINGS}")
endif(CMAKE_COMPILER_IS_GNUCXX)
